import 'package:flutter/material.dart';
import 'package:get/get.dart';

import 'package:tosspayments_sdk_flutter/model/paymentData.dart';

class Home extends StatefulWidget {
  @override
  _HomeState createState() => _HomeState();
}

class _HomeState extends State<Home> {
  final _form = GlobalKey<FormState>();
  late String payMethod; // 결제수단
  late String orderId; // 주문번호
  late String orderName; // 주문명
  late String amount; // 결제금액
  late String customerName; // 주문자명
  late String customerEmail; // 구매자 이메일

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: AppBar(
        title: Text('toss payments 결제 테스트'),
        centerTitle: true,

      ),
      body: SafeArea(
        minimum: EdgeInsets.symmetric(horizontal: 15),
        child: Form(
          key: _form,
          child: ListView(
            children: [
              DropdownButtonFormField<String?>(
                decoration: InputDecoration(
                  labelText: '결제수단',
                  floatingLabelBehavior: FloatingLabelBehavior.always,
                  labelStyle: TextStyle(fontSize: 15, color: Color(0xffcfcfcf)),
                ),
                onChanged: (String? newValue) {
                  payMethod = newValue!;
                },
                items:
                [null, '카드','가상계좌','계좌이체','휴대폰','상품권'].map<DropdownMenuItem<String?>>((String? i) {
                  return DropdownMenuItem<String?>(
                    value: i,
                    child: Text({'카드': '카드', '가상계좌': '가상계좌','계좌이체':'계좌이체','휴대폰':'휴대폰','상품권':'상품권'}[i] ?? '결제수단 선택'),
                  );
                }).toList(),
              ),
              TextFormField(
                decoration: InputDecoration(
                  labelText: '주문번호(orderId)',
                ),
                initialValue: 'tosspaymentsFlutter_${DateTime.now().millisecondsSinceEpoch}',
                onSaved: (String? value) {
                  orderId = value!;
                },
              ),
              TextFormField(
                decoration: InputDecoration(
                  labelText: '주문명(orderName)',
                ),
                initialValue: 'Toss T-shirt',
                onSaved: (String? value) {
                  orderName = value!;
                },
              ),
              TextFormField(
                decoration: InputDecoration(
                  labelText: '결제금액(amount)',
                ),
                initialValue: '50000',
                keyboardType: TextInputType.numberWithOptions(decimal: true),
                onSaved: (String? value) {
                  amount = value!;
                },
              ),

              TextFormField(
                decoration: InputDecoration(
                  labelText: '구매자명(customerName)',
                ),
                initialValue: '김토스',
                onSaved: (String? value) {
                  customerName = value!;
                },
              ),

              TextFormField(
                decoration: InputDecoration(
                  labelText: '이메일(customerEmail)',
                ),
                initialValue: 'tosspayments@toss-payments.co.kr',
                keyboardType: TextInputType.emailAddress,
                onSaved: (String? value) {
                  customerEmail = value!;
                },
              ),
              Container(
                  padding: EdgeInsets.symmetric(vertical: 10),
                  child: ElevatedButton(
                    onPressed: () {
                      _form.currentState!.save();
                      PaymentData data = PaymentData(
                          paymentMethod: payMethod,
                          orderId: orderId,
                          orderName: orderName,
                          amount: int.parse(amount),
                          customerName: customerName,
                          customerEmail: customerEmail
                      );
                      Get.toNamed('/payment', arguments: data);
                    }, child:  Text(
                    '결제하기',
                    style: TextStyle(
                      fontSize: 18,
                      color: Colors.white,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                  )
              )
            ],
          ),
        ),
      ),
    );
  }
}
