import 'dart:convert';
import 'dart:io';

import 'package:flutter/foundation.dart';
import 'package:flutter/gestures.dart';
import 'package:flutter/material.dart';
import 'package:tosspayments_sdk_flutter/model/tosspayments_url.dart';
import 'package:webview_flutter/webview_flutter.dart';

enum ActionType { auth, payment }

class TosspaymentsWebview extends StatefulWidget {
  static final String html = '''
    <html>
      <head>
        <meta http-equiv="content-type" content="text/html; charset=utf-8">
        <meta name="viewport" content="width=device-width, initial-scale=1.0, user-scalable=no">
        <script src="https://js.tosspayments.com/v1/payment"></script>
      </head>
      <body>      
      </body>
    </html>
  ''';

  final ActionType type;
  final PreferredSizeWidget? appBar;
  final Widget? initialChild;
  final ValueSetter<WebViewController> executeJS;
  final ValueSetter<Map<String, String>> getQueryParams;
  final Function ifAuthDone;
  final Set<Factory<OneSequenceGestureRecognizer>>? gestureRecognizers;

  TosspaymentsWebview({
    required this.type,
    this.appBar,
    this.initialChild,
    required this.executeJS,
    required this.getQueryParams,
    required this.ifAuthDone,
    this.gestureRecognizers,
  });

  @override
  _TosspaymentsWebviewState createState() => _TosspaymentsWebviewState();
}

class _TosspaymentsWebviewState extends State<TosspaymentsWebview> {
  late WebViewController _webViewController;
  late int _isWebviewLoaded;
  late int _isPageLoaded;

  @override
  void initState() {
    super.initState();
    _isWebviewLoaded = 0;
    _isPageLoaded = 0;
    if (Platform.isAndroid) {
      WebView.platform = SurfaceAndroidWebView();
    }
    if (widget.initialChild != null) {
      _isWebviewLoaded++;
    }
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: widget.appBar,
      body: SafeArea(
        child: IndexedStack(
          index: _isWebviewLoaded,
          children: [
            WebView(
              initialUrl:
              Uri.dataFromString(TosspaymentsWebview.html, mimeType: 'text/html', encoding: Encoding.getByName('utf-8'))
                  .toString(),
              javascriptMode: JavascriptMode.unrestricted,
              gestureRecognizers: widget.gestureRecognizers,
              onWebViewCreated: (controller) {
                _webViewController = controller;
              },
              onPageFinished: (String url) {
                // 웹뷰 로딩 완료시에 화면 전환
                if (_isWebviewLoaded == 1) {
                  setState(() {
                    _isWebviewLoaded = 0;
                  });
                }

                // 페이지 로딩 완료시 JS 실행
                if (_isPageLoaded == 0) {
                  widget.executeJS(_webViewController);
                  _isPageLoaded++;
                }
              },

              navigationDelegate: (request) async {  // 웹뷰 화면 전환 시 실행
                //원천사 인증 완료시
                if (widget.ifAuthDone(request.url)) {
                  String decodedUrl = Uri.decodeComponent(request.url);
                  widget.getQueryParams(Uri.parse(decodedUrl).queryParameters);
                  return NavigationDecision.prevent;
                }

                final convertUrl = ConvertUrl(request.url);

                // 앱링크일시 convert하기 위해
                if (convertUrl.isAppLink()) {
                  convertUrl.launchApp();
                  return NavigationDecision.prevent;
                }

                return NavigationDecision.navigate;
              },
            ),
            if (_isWebviewLoaded == 1) widget.initialChild!,
          ],
        ),
      ),
    );
  }
}
